"""Build optional cython modules."""

import contextlib
import os
from distutils.command.build_ext import build_ext
from os.path import join
from typing import Any

try:
    from setuptools import Extension
except ImportError:
    from distutils.core import Extension

kasa_crypt_module = Extension(
    "kasa_crypt._crypt_impl",
    [
        join("src", "kasa_crypt", "_crypt_impl.pyx"),
    ],
    language="c",
    extra_compile_args=["-O3", "-g0"],
)


class BuildExt(build_ext):
    def build_extensions(self) -> None:
        with contextlib.suppress(Exception):
            super().build_extensions()


def build(setup_kwargs: dict[Any, Any]) -> None:
    if os.environ.get("SKIP_CYTHON", False):
        return
    try:
        from Cython.Build import cythonize

        setup_kwargs.update(
            dict(
                ext_modules=cythonize(
                    [
                        kasa_crypt_module,
                    ],
                    compiler_directives={"language_level": "3"},  # Python 3
                ),
                cmdclass=dict(build_ext=BuildExt),
            )
        )
        setup_kwargs["exclude_package_data"] = {
            pkg: ["*.c"] for pkg in setup_kwargs["packages"]
        }
    except Exception:
        if os.environ.get("REQUIRE_CYTHON"):
            raise
        pass



def encrypt_pure_python(unencrypted: bytes) -> bytearray:
    """Encrypt."""
    key = 171
    unencrypted_len = len(unencrypted)
    encrypted = bytearray(unencrypted_len)
    for idx, unencryptedbyte in enumerate(unencrypted):
        key = key ^ unencryptedbyte
        encrypted[idx] = key
    return encrypted


def decrypt_pure_python(string: bytes) -> str:
    """Decrypt."""
    key = 171
    result = bytearray(len(string))
    for idx, i in enumerate(string):
        a = key ^ i
        key = i
        result[idx] = a
    return result.decode()



import cython

from libc.stdlib cimport free, malloc
from libc.string cimport strlen


cdef extern from "crypt_wrapper.h":
    void _encrypt_into(const char * unencrypted, char * encrypted, unsigned long length)
    void _decrypt_into(const char * encrypted, char * unencrypted, unsigned long length)

cdef void _decrypt(const char *encrypted, char **unencrypted, Py_ssize_t length):
    unencrypted[0] = <char *> malloc((length + 1) * sizeof(char))
    if not unencrypted[0]:
        return  # malloc failed
    _decrypt_into(encrypted, unencrypted[0], length)

cdef void _encrypt(const char *unencrypted, char** encrypted, Py_ssize_t length):
    encrypted[0] = <char *> malloc((length + 1) * sizeof(char))
    if not encrypted[0]:
        return  # malloc failed
    _encrypt_into(unencrypted, encrypted[0], length)

def encrypt(py_byte_string: bytes) -> bytes:
    cdef char* encrypted = NULL
    cdef Py_ssize_t length = len(py_byte_string)
    _encrypt(py_byte_string, &encrypted, length)
    try:
        return encrypted[:length]
    finally:
        free(encrypted)

def decrypt(string: bytes) -> str:
    cdef char* unencrypted = NULL
    cdef Py_ssize_t length = len(string)
    _decrypt(string, &unencrypted, length)
    try:
        return unencrypted[:length].decode('utf-8')
    finally:
        free(unencrypted)



__version__ = "0.6.1"

from struct import Struct

_pack_header = Struct(">I").pack

try:
    from ._crypt_impl import decrypt
    from ._crypt_impl import encrypt as _encrypt
except ImportError:
    from .python_impl import decrypt_pure_python as decrypt
    from .python_impl import encrypt_pure_python as _encrypt


def encrypt(string: str) -> bytes:
    """Encrypt."""
    encoded = string.encode("utf-8")
    return _pack_header(len(encoded)) + _encrypt(encoded)


__all__ = ["encrypt", "decrypt"]
